package com.example.switchyard.tests;
/* 
 * JBoss, Home of Professional Open Source 
 * Copyright 2010-2011 Red Hat Inc. and/or its affiliates and other contributors
 * as indicated by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a 
 * full listing of individual contributors.
 *
 * This copyrighted material is made available to anyone wishing to use, 
 * modify, copy, or redistribute it subject to the terms and conditions 
 * of the GNU Lesser General Public License, v. 2.1. 
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more 
 * details. You should have received a copy of the GNU Lesser General Public 
 * License, v.2.1 along with this distribution; if not, write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301, USA.
 */
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

import org.switchyard.annotations.Transformer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.io.StringReader;

public class Transformers {

    // Element names in XML document
//    private static final String ORDER_ID = "orderId";
//    private static final String ITEM_ID = "itemId";
//    private static final String QUANTITY = "quantity";
//    
    private static final String VALUES = "values";
    private static final String USRID = "usrId";
    private static final String PASSWORD = "password";
    private static final String COMPANYID = "companyId";
    private static final String LANGUAGE = "language";
    
//lookupItem
	@Transformer(to = "{urn:com.example.switchyard:tests:1.0}equipmentSearch")
	public String transformStringToLookupItem(String from) {
		return "<urn:equipmentSearch xmlns:urn=\"urn:com.example.switchyard:tests:1.0\"><string>" + from + "</string></urn:equipmentSearch>";
	}

	@Transformer(from = "{urn:com.example.switchyard:tests:1.0}equipmentSearchResponse")
	public Item transformLookupItemResponseToItem(String from) {
		return new Item().set_values("");
	}

    /**
     * Transform from a DOM element to an Order instance.
     * <p/>
     * No need to specify the "to" type because Order is a concrete type.
     * @param from Order element.
     * @return Order instance.
     */
    @Transformer(from = "{urn:com.example.switchyard:tests:1.0}equipmentSearch")
    public Order transform(Element from) {
       return new Order()
       		.set_usrId(getElementValue(from,USRID))
           .set_pass(getElementValue(from, PASSWORD))
           .setCompanyId(getElementValue(from, COMPANYID))
           .set_values(getElementValue(from, VALUES))
           .set_language(getElementValue(from, LANGUAGE));
    }

    /**
     * Transform from an OrderAck to an Element.
     * <p/>
     * No need to specify the "from" type because OrderAck is a concrete type.
     * @param orderAck OrderAck.
     * @return Order response element.
     */
    @Transformer(to = "{urn:com.example.switchyard:tests:1.0}equipmentSearchResponse")
    public Element transform(OrderAck orderAck) {
        StringBuffer ackXml = new StringBuffer()
            .append("<orders:submitOrderResponse xmlns:orders=\"urn:com.example.switchyard:tests:1.0\">")
            .append("<orderAck>")
            .append("<usrId>" + orderAck.get_usrId() + "</usrId>")
            .append("<password>" + orderAck.get_pass() + "</password>")
            .append("<companyId>" + orderAck.getCompanyId() + "</companyId>")
            .append("<language>" + orderAck.get_language() + "</language>")
            .append("<values>" + orderAck.get_values()+ "</values>")
            
//            .append("<orderId>" + orderAck.getOrderId() + "</orderId>")
//            .append("<accepted>" + orderAck.isAccepted() + "</accepted>")
//            .append("<status>" + orderAck.getStatus() + "</status>")
            .append("</orderAck>")
            .append("</orders:submitOrderResponse>");

        return toElement(ackXml.toString());
    }

    /**
     * Returns the text value of a child node of parent.
     */
    private String getElementValue(Element parent, String elementName) {
        String value = null;
        NodeList nodes = parent.getElementsByTagName(elementName);
        if (nodes.getLength() > 0) {
            value = nodes.item(0).getChildNodes().item(0).getNodeValue();
        }
        return value;
    }

    private Element toElement(String xml) {
        DOMResult dom = new DOMResult();
        try {
            TransformerFactory.newInstance().newTransformer().transform(
                    new StreamSource(new StringReader(xml)), dom);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        return ((Document)dom.getNode()).getDocumentElement();
    }
}



